/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */


package it.dis.uniroma1.jamote.core.language.java;


import java.lang.reflect.Method;
import java.util.List;

import it.dis.uniroma1.jamote.core.language.Function;
import java.util.ArrayList;


public class JavaFunction implements Function {
    
	protected Method javaMethod;
        
	public JavaFunction(Method m){
		this.javaMethod = m;
    }
        
	public String getDeclaringClass(){
		return cleanUpTypeName(javaMethod.getDeclaringClass());
	}
	
    public int getModifiers(){
		return javaMethod.getModifiers();
    }
	
	public String getName(){
		return javaMethod.getName();
	}
	
	public String getReturnType(){
		return cleanUpTypeName(javaMethod.getReturnType());
	}
	
	public List<String> getParameterTypes(){
		List<String> parameterList = new ArrayList<String>();
                Class<?> parameters[] = javaMethod.getParameterTypes();
        for(int i = 0; i < parameters.length; i++){
        	parameterList.add(cleanUpTypeName(parameters[i]));
        }
        
		return parameterList;
    }
	
    private String cleanUpTypeName(Class<?> c){
    	String[] temp = c.toString().split("\\s");
    	if(temp[0].equals("class")){
        	if(temp[1].startsWith("[I")) 
        		return("int[]");
        	else
        		return temp[1];
        }
        else 
        	return temp[0];
        
    }

	public List<String> getExceptionTypes() {
		List<String> excList = new ArrayList<String>();
        Class<?> excs[] = javaMethod.getExceptionTypes();
		for(int i = 0; i < excs.length; i++){
			String[] excName = excs[i].toString().split("\\s|\\.");
			excList.add(excName[excName.length - 1]);
		}
		
		return excList;
	}
}
