/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */


package it.dis.uniroma1.jamote.core.language.java;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import it.dis.uniroma1.jamote.core.language.Function;
import it.dis.uniroma1.jamote.core.language.JamoteLanguageException;
import it.dis.uniroma1.jamote.core.language.Tester;
import it.dis.uniroma1.jamote.core.modeling.InputModel;
import it.dis.uniroma1.jamote.core.modeling.OutputModel;
import it.dis.uniroma1.jamote.utils.Reporter;



public class JavaTester extends Tester {
	
	public JavaTester(Reporter reporter){
		super(reporter);
	}
	
	/**
	 * Esegue il test sulla funzione specificata
	 */
	public OutputModel executeTest(Function function, InputModel input) throws JamoteLanguageException{
		try{
			Class<?> c = Class.forName(function.getDeclaringClass());
			
			//Genero i tipi di dato degli argomenti per cercare la funzione da testare
			Class<?> params[] = new Class<?>[function.getParameterTypes().size()];
			int i = 0;
			for(String parameterType: function.getParameterTypes()){
				if(parameterType.equals("int"))
					params[i] = int.class;
				else if(parameterType.equals("int[]"))
					params[i] = int[].class;
				else
					throw new JamoteLanguageException("Unsupported parameter type: " + parameterType);
				i++;
			}
			
			Method method = c.getMethod(function.getName(), params);
			
			//L'oggetto di invocazione
			Object invokationObject = c.newInstance();
			
			//creo l'array di argomenti, concretizzazione finale (immediata Java->Java)
			Object[] arguments = new Object[input.getObjects().size()];
			for(i = 0; i < input.getObjects().size(); i++)
				arguments[i] = input.getObjects().get(new Integer(i)).getValue();
			
			try{
				Object ret = method.invoke(invokationObject, arguments);
				return abstractObject(ret, false);
			}
			catch(InvocationTargetException ex){
				return abstractObject(ex.getCause(), true);
			}
		}
		catch(ClassNotFoundException ex){
			throw new JamoteLanguageException("Test class not found: " + function.getDeclaringClass(), ex);
		}
		catch(NoSuchMethodException ex){
			throw new JamoteLanguageException("Test method not found: " + function.getName(), ex);
		}
		catch(Exception ex){
			throw new JamoteLanguageException("Method invocation error", ex);
		}
	}

	
	/**
	 * 
	 * @param obj
	 * @param throwed
	 * @return
	 * @throws JamoteLanguageException
	 */
	protected OutputModel abstractObject(Object obj, boolean throwed) throws JamoteLanguageException {
		if(throwed)
		{
			String[] excName = obj.getClass().toString().split("\\s|\\.");
			return new OutputModel(obj.toString(), excName[excName.length - 1], obj, true);
		}
		
		if(obj instanceof Integer){
			return new OutputModel(obj.toString(), "int", obj);
		}
		else if(obj instanceof int[]){
			return new OutputModel("int[]", "int[]", obj);
		}
		else
			throw new JamoteLanguageException("Unsupported abstraction type: " + obj.getClass());
		
	}
}
