/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */

package it.dis.uniroma1.jamote.core.modeling;

import java.util.HashMap;
import java.util.Map;


/**
 * 
 * @author marius
 *
 */
public class AbstractObject {
	protected String label;
	protected String type;
	protected Object value;
	protected Map<String,Object> fields;
	
	
	public AbstractObject(String label, String type, Object value) {
		this.label = label;
		this.type = type;
		this.value = value;
		this.fields = new HashMap<String, Object>();
	}
	
	
	public Map<String, Object> getFields() {
		return fields;
	}

	public String getLabel() {
		return label;
	}


	public String getType() {
		return type;
	}


	public Object getValue() {
		return value;
	}

	@Override
	public String toString() {
		if(value instanceof int[]){
			String ret = "int[] = {";
			int[] arr = (int[]) value;
			for(int v: arr)
				ret += v + " ";
			
			ret += "}";
			
			return ret;
		}
		return "" + value;
	}


	@Override
	public boolean equals(Object obj) {
		if(! (obj instanceof AbstractObject))
			return false;
		
		AbstractObject obj2 = (AbstractObject) obj;
		if(obj2.getValue() == null) return this.getValue() == null;
		if(this.getValue() == null) return obj2.getValue() == null;
		
		if(this.label != obj2.label || !this.type.equals(obj2.type))
			return false;
		
		if(this.type.equals("int"))
			return this.value == obj2.value;
		if(this.type.equals("int[]")){
			int[] value1 = (int[]) this.value;
			int[] value2 = (int[]) obj2.value;
			if(value1.length != value2.length)
				return false;
			
			for(int i=0; i < value1.length; i++)
				if(value1[i] != value2[i])
					return false;
			
			return true;
		}
		
		return  false;
	}
}
