/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */


/*
 * DesktopApplication1View.java
 */

package it.dis.uniroma1.jamote.gui;

import it.dis.uniroma1.jamote.core.EquivalencePartition;
import it.dis.uniroma1.jamote.core.JamoteException;
import it.dis.uniroma1.jamote.core.Test;
import it.dis.uniroma1.jamote.core.language.Function;
import it.dis.uniroma1.jamote.utils.Config;
import it.dis.uniroma1.jamote.utils.Reporter;
import it.dis.uniroma1.jamote.utils.ReportEntry.Level;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.lang.String;
import java.util.ArrayList;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;

/**
 * The application's main frame.
 */
public class BlackBoxTestGuiView extends FrameView{
	
	

    public BlackBoxTestGuiView(SingleFrameApplication app) {
        super(app);

        initComponents();
		this.getFrame().setResizable(false);
		
        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        //statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
		
		tabbedPane.setSelectedIndex(0);
        outputTextPane.setEditable(false);
        methodSignatureList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		statusAnimationLabel.setIcon(idleIcon);
		}
		
		
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = BlackBoxTestGui.getApplication().getMainFrame();
            aboutBox = new BlackBoxTestGuiAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        BlackBoxTestGui.getApplication().show(aboutBox);
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        openMenuItem = new javax.swing.JMenuItem();
        openClassInPackageMenuItem = new javax.swing.JMenuItem();
        saveAlloyFileMenuItem = new javax.swing.JMenuItem();
        openAlloyFileMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        tabbedPane = new javax.swing.JTabbedPane();
        InputPanel = new javax.swing.JPanel();
        extractMethodsPanel = new javax.swing.JPanel();
        methodSignatureScrollPane = new javax.swing.JScrollPane();
        methodSignatureList = new javax.swing.JList();
        sourceCodePanel = new javax.swing.JPanel();
        codiceEditorScrollPane = new javax.swing.JScrollPane();
        codiceEditorPane = new javax.swing.JEditorPane();
        jButton2 = new javax.swing.JButton();
        preconditionPanel = new javax.swing.JPanel();
        insertPreconditionPanel = new javax.swing.JPanel();
        insertPreconditionScrollPane = new javax.swing.JScrollPane();
        preconditionEditorPane = new javax.swing.JEditorPane();
        clearPreconditionButton = new javax.swing.JButton();
        selectedMethodNameLabel = new javax.swing.JLabel();
        selectedMethodLabel = new javax.swing.JLabel();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        functionPanel = new javax.swing.JPanel();
        selectedMethodLabel4 = new javax.swing.JLabel();
        selectedMethodNameLabel4 = new javax.swing.JLabel();
        insertFunctionPanel = new javax.swing.JPanel();
        functionScrollPane = new javax.swing.JScrollPane();
        functionEditorPane = new javax.swing.JEditorPane();
        clearFunctionButton = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();
        equivalencePartitionPanel = new javax.swing.JPanel();
        deleteEquivalencePartitionButton = new javax.swing.JButton();
        modifyEquivalencePartitionButton = new javax.swing.JButton();
        equivalencePartitionInputPanel = new javax.swing.JPanel();
        equivalencePartitionInputScrollPane = new javax.swing.JScrollPane();
        equivalencePartitionInputEditorPane = new javax.swing.JEditorPane();
        insertedEquivalencePartitionPanel = new javax.swing.JPanel();
        insertedEquivalencePartitionScrollPane = new javax.swing.JScrollPane();
        equivalencePartitionList = new javax.swing.JList();
        selectedMethodNameLabel2 = new javax.swing.JLabel();
        selectedMethodLabel2 = new javax.swing.JLabel();
        maxModelLabel = new javax.swing.JLabel();
        maxDimensionTextField = new javax.swing.JTextField();
        testForModelLabel = new javax.swing.JLabel();
        testForModelTextField = new javax.swing.JTextField();
        TestButton = new javax.swing.JButton();
        equivalencePartitionOutputPanel = new javax.swing.JPanel();
        equivalencePartitionOutputScrollPane = new javax.swing.JScrollPane();
        equivalencePartitionOutputEditorPane = new javax.swing.JEditorPane();
        bitwidthLabel = new javax.swing.JLabel();
        bitwidthTextField = new javax.swing.JTextField();
        addEquivalencePartitionButton = new javax.swing.JButton();
        jButton7 = new javax.swing.JButton();
        jButton8 = new javax.swing.JButton();
        equivalencePartitionNameLabel = new javax.swing.JLabel();
        equivalencePartitionNameTextField = new javax.swing.JTextField();
        testPanel = new javax.swing.JPanel();
        selectedMethodNameLabel3 = new javax.swing.JLabel();
        ReportEntryComboBox = new javax.swing.JComboBox();
        outputPanel = new javax.swing.JPanel();
        outputScrollPane = new javax.swing.JScrollPane();
        outputTextPane = new javax.swing.JTextPane();
        selectedMethodLabel3 = new javax.swing.JLabel();
        outputMessageLevelLabel = new javax.swing.JLabel();
        jButton9 = new javax.swing.JButton();
        stopTestButton = new javax.swing.JButton();

        menuBar.setName("menuBar"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(it.dis.uniroma1.jamote.gui.BlackBoxTestGui.class).getContext().getResourceMap(BlackBoxTestGuiView.class);
        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        openMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        openMenuItem.setText(resourceMap.getString("openMenuItem.text")); // NOI18N
        openMenuItem.setName("openMenuItem"); // NOI18N
        openMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(openMenuItem);

        openClassInPackageMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.CTRL_MASK));
        openClassInPackageMenuItem.setText(resourceMap.getString("openClassInPackageMenuItem.text")); // NOI18N
        openClassInPackageMenuItem.setName("openClassInPackageMenuItem"); // NOI18N
        openClassInPackageMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openClassInPackageMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(openClassInPackageMenuItem);

        saveAlloyFileMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        saveAlloyFileMenuItem.setText(resourceMap.getString("saveAlloyFileMenuItem.text")); // NOI18N
        saveAlloyFileMenuItem.setName("saveAlloyFileMenuItem"); // NOI18N
        saveAlloyFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAlloyFileMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(saveAlloyFileMenuItem);

        openAlloyFileMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, java.awt.event.InputEvent.CTRL_MASK));
        openAlloyFileMenuItem.setText(resourceMap.getString("openAlloyFileMenuItem.text")); // NOI18N
        openAlloyFileMenuItem.setName("openAlloyFileMenuItem"); // NOI18N
        openAlloyFileMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openAlloyFileMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(openAlloyFileMenuItem);

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(it.dis.uniroma1.jamote.gui.BlackBoxTestGui.class).getContext().getActionMap(BlackBoxTestGuiView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setMaximum(100000);
        progressBar.setName("progressBar"); // NOI18N

        tabbedPane.setAutoscrolls(true);
        tabbedPane.setMinimumSize(new java.awt.Dimension(0, 0));
        tabbedPane.setName("tabbedPane"); // NOI18N

        InputPanel.setName("InputPanel"); // NOI18N

        extractMethodsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("extractMethodsPanel.border.title"))); // NOI18N
        extractMethodsPanel.setName("extractMethodsPanel"); // NOI18N

        methodSignatureScrollPane.setName("methodSignatureScrollPane"); // NOI18N

        methodSignatureList.setMinimumSize(new java.awt.Dimension(50, 5));
        methodSignatureList.setName("methodSignatureList"); // NOI18N
        methodSignatureList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                methodSignatureListMousePressed(evt);
            }
        });
        methodSignatureScrollPane.setViewportView(methodSignatureList);

        javax.swing.GroupLayout extractMethodsPanelLayout = new javax.swing.GroupLayout(extractMethodsPanel);
        extractMethodsPanel.setLayout(extractMethodsPanelLayout);
        extractMethodsPanelLayout.setHorizontalGroup(
            extractMethodsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(methodSignatureScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 606, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        extractMethodsPanelLayout.setVerticalGroup(
            extractMethodsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(methodSignatureScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE)
        );

        sourceCodePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("sourceCodePanel.border.title"))); // NOI18N
        sourceCodePanel.setName("sourceCodePanel"); // NOI18N

        codiceEditorScrollPane.setName("codiceEditorScrollPane"); // NOI18N

        codiceEditorPane.setEditable(false);
        codiceEditorPane.setName("codiceEditorPane"); // NOI18N
        codiceEditorScrollPane.setViewportView(codiceEditorPane);

        javax.swing.GroupLayout sourceCodePanelLayout = new javax.swing.GroupLayout(sourceCodePanel);
        sourceCodePanel.setLayout(sourceCodePanelLayout);
        sourceCodePanelLayout.setHorizontalGroup(
            sourceCodePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(codiceEditorScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 698, Short.MAX_VALUE)
        );
        sourceCodePanelLayout.setVerticalGroup(
            sourceCodePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(codiceEditorScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 283, Short.MAX_VALUE)
        );

        jButton2.setIcon(resourceMap.getIcon("jButton2.icon")); // NOI18N
        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setToolTipText(resourceMap.getString("jButton2.toolTipText")); // NOI18N
        jButton2.setBorderPainted(false);
        jButton2.setContentAreaFilled(false);
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout InputPanelLayout = new javax.swing.GroupLayout(InputPanel);
        InputPanel.setLayout(InputPanelLayout);
        InputPanelLayout.setHorizontalGroup(
            InputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(InputPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(InputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(extractMethodsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(sourceCodePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(27, 27, 27))
        );
        InputPanelLayout.setVerticalGroup(
            InputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(InputPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(sourceCodePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(extractMethodsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        tabbedPane.addTab(resourceMap.getString("InputPanel.TabConstraints.tabTitle"), InputPanel); // NOI18N

        preconditionPanel.setName("preconditionPanel"); // NOI18N

        insertPreconditionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("insertPreconditionPanel.border.title"))); // NOI18N
        insertPreconditionPanel.setName("insertPreconditionPanel"); // NOI18N

        insertPreconditionScrollPane.setName("insertPreconditionScrollPane"); // NOI18N

        preconditionEditorPane.setBorder(null);
        preconditionEditorPane.setName("preconditionEditorPane"); // NOI18N
        insertPreconditionScrollPane.setViewportView(preconditionEditorPane);

        clearPreconditionButton.setFont(resourceMap.getFont("clearPreconditionButton.font")); // NOI18N
        clearPreconditionButton.setText(resourceMap.getString("clearPreconditionButton.text")); // NOI18N
        clearPreconditionButton.setName("clearPreconditionButton"); // NOI18N
        clearPreconditionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearPreconditionButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout insertPreconditionPanelLayout = new javax.swing.GroupLayout(insertPreconditionPanel);
        insertPreconditionPanel.setLayout(insertPreconditionPanelLayout);
        insertPreconditionPanelLayout.setHorizontalGroup(
            insertPreconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(insertPreconditionPanelLayout.createSequentialGroup()
                .addComponent(insertPreconditionScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 606, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(clearPreconditionButton, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        insertPreconditionPanelLayout.setVerticalGroup(
            insertPreconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(insertPreconditionScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 513, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, insertPreconditionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(clearPreconditionButton)
                .addContainerGap())
        );

        selectedMethodNameLabel.setText(resourceMap.getString("selectedMethodNameLabel.text")); // NOI18N
        selectedMethodNameLabel.setName("selectedMethodNameLabel"); // NOI18N

        selectedMethodLabel.setForeground(resourceMap.getColor("selectedMethodLabel.foreground")); // NOI18N
        selectedMethodLabel.setText(resourceMap.getString("selectedMethodLabel.text")); // NOI18N
        selectedMethodLabel.setName("selectedMethodLabel"); // NOI18N

        jButton3.setIcon(resourceMap.getIcon("jButton3.icon")); // NOI18N
        jButton3.setBorder(null);
        jButton3.setBorderPainted(false);
        jButton3.setContentAreaFilled(false);
        jButton3.setName("jButton3"); // NOI18N
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jButton4.setIcon(resourceMap.getIcon("jButton4.icon")); // NOI18N
        jButton4.setToolTipText(resourceMap.getString("jButton4.toolTipText")); // NOI18N
        jButton4.setBorderPainted(false);
        jButton4.setContentAreaFilled(false);
        jButton4.setName("jButton4"); // NOI18N

        javax.swing.GroupLayout preconditionPanelLayout = new javax.swing.GroupLayout(preconditionPanel);
        preconditionPanel.setLayout(preconditionPanelLayout);
        preconditionPanelLayout.setHorizontalGroup(
            preconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(preconditionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(preconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(preconditionPanelLayout.createSequentialGroup()
                        .addComponent(selectedMethodLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 111, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(selectedMethodNameLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 329, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jButton3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton4, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(insertPreconditionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(29, Short.MAX_VALUE))
        );
        preconditionPanelLayout.setVerticalGroup(
            preconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(preconditionPanelLayout.createSequentialGroup()
                .addGroup(preconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton4, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(preconditionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(selectedMethodNameLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(selectedMethodLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(insertPreconditionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(31, Short.MAX_VALUE))
        );

        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        tabbedPane.addTab(resourceMap.getString("preconditionPanel.TabConstraints.tabTitle"), preconditionPanel); // NOI18N

        functionPanel.setName("functionPanel"); // NOI18N

        selectedMethodLabel4.setForeground(resourceMap.getColor("selectedMethodLabel4.foreground")); // NOI18N
        selectedMethodLabel4.setText(resourceMap.getString("selectedMethodLabel4.text")); // NOI18N
        selectedMethodLabel4.setName("selectedMethodLabel4"); // NOI18N

        selectedMethodNameLabel4.setName("selectedMethodNameLabel4"); // NOI18N

        insertFunctionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("insertFunctionPanel.border.title"))); // NOI18N
        insertFunctionPanel.setName("insertFunctionPanel"); // NOI18N

        functionScrollPane.setName("functionScrollPane"); // NOI18N

        functionEditorPane.setBorder(null);
        functionEditorPane.setName("functionEditorPane"); // NOI18N
        functionScrollPane.setViewportView(functionEditorPane);

        clearFunctionButton.setFont(resourceMap.getFont("clearFunctionButton.font")); // NOI18N
        clearFunctionButton.setText(resourceMap.getString("clearFunctionButton.text")); // NOI18N
        clearFunctionButton.setName("clearFunctionButton"); // NOI18N
        clearFunctionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearFunctionButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout insertFunctionPanelLayout = new javax.swing.GroupLayout(insertFunctionPanel);
        insertFunctionPanel.setLayout(insertFunctionPanelLayout);
        insertFunctionPanelLayout.setHorizontalGroup(
            insertFunctionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(insertFunctionPanelLayout.createSequentialGroup()
                .addComponent(functionScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 605, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(clearFunctionButton, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        insertFunctionPanelLayout.setVerticalGroup(
            insertFunctionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(functionScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 518, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, insertFunctionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(clearFunctionButton)
                .addContainerGap())
        );

        jButton5.setIcon(resourceMap.getIcon("jButton5.icon")); // NOI18N
        jButton5.setBorder(null);
        jButton5.setBorderPainted(false);
        jButton5.setContentAreaFilled(false);
        jButton5.setName("jButton5"); // NOI18N

        jButton6.setIcon(resourceMap.getIcon("jButton6.icon")); // NOI18N
        jButton6.setToolTipText(resourceMap.getString("jButton6.toolTipText")); // NOI18N
        jButton6.setBorderPainted(false);
        jButton6.setContentAreaFilled(false);
        jButton6.setName("jButton6"); // NOI18N

        javax.swing.GroupLayout functionPanelLayout = new javax.swing.GroupLayout(functionPanel);
        functionPanel.setLayout(functionPanelLayout);
        functionPanelLayout.setHorizontalGroup(
            functionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(functionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(functionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(functionPanelLayout.createSequentialGroup()
                        .addComponent(insertFunctionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())
                    .addGroup(functionPanelLayout.createSequentialGroup()
                        .addComponent(selectedMethodLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 111, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(selectedMethodNameLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 298, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 237, Short.MAX_VALUE)
                        .addComponent(jButton5, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton6, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(30, 30, 30))))
        );
        functionPanelLayout.setVerticalGroup(
            functionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(functionPanelLayout.createSequentialGroup()
                .addGroup(functionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(functionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addGroup(functionPanelLayout.createSequentialGroup()
                            .addContainerGap()
                            .addGroup(functionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jButton6, javax.swing.GroupLayout.Alignment.TRAILING, 0, 0, Short.MAX_VALUE)
                                .addComponent(jButton5, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                        .addComponent(selectedMethodLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, 37, Short.MAX_VALUE))
                    .addComponent(selectedMethodNameLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, 37, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(insertFunctionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(29, Short.MAX_VALUE))
        );

        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        tabbedPane.addTab(resourceMap.getString("functionPanel.TabConstraints.tabTitle"), functionPanel); // NOI18N

        equivalencePartitionPanel.setName("equivalencePartitionPanel"); // NOI18N

        deleteEquivalencePartitionButton.setFont(resourceMap.getFont("deleteEquivalencePartitionButton.font")); // NOI18N
        deleteEquivalencePartitionButton.setText(resourceMap.getString("deleteEquivalencePartitionButton.text")); // NOI18N
        deleteEquivalencePartitionButton.setName("deleteEquivalencePartitionButton"); // NOI18N
        deleteEquivalencePartitionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteEquivalencePartitionButtonActionPerformed(evt);
            }
        });

        modifyEquivalencePartitionButton.setFont(resourceMap.getFont("modifyEquivalencePartitionButton.font")); // NOI18N
        modifyEquivalencePartitionButton.setText(resourceMap.getString("modifyEquivalencePartitionButton.text")); // NOI18N
        modifyEquivalencePartitionButton.setName("modifyEquivalencePartitionButton"); // NOI18N
        modifyEquivalencePartitionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modifyEquivalencePartitionButtonActionPerformed(evt);
            }
        });

        equivalencePartitionInputPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("equivalencePartitionInputPanel.border.title"))); // NOI18N
        equivalencePartitionInputPanel.setName("equivalencePartitionInputPanel"); // NOI18N

        equivalencePartitionInputScrollPane.setName("equivalencePartitionInputScrollPane"); // NOI18N

        equivalencePartitionInputEditorPane.setName("equivalencePartitionInputEditorPane"); // NOI18N
        equivalencePartitionInputScrollPane.setViewportView(equivalencePartitionInputEditorPane);

        javax.swing.GroupLayout equivalencePartitionInputPanelLayout = new javax.swing.GroupLayout(equivalencePartitionInputPanel);
        equivalencePartitionInputPanel.setLayout(equivalencePartitionInputPanelLayout);
        equivalencePartitionInputPanelLayout.setHorizontalGroup(
            equivalencePartitionInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(equivalencePartitionInputScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 603, Short.MAX_VALUE)
        );
        equivalencePartitionInputPanelLayout.setVerticalGroup(
            equivalencePartitionInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(equivalencePartitionInputScrollPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 129, Short.MAX_VALUE)
        );

        insertedEquivalencePartitionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("insertedEquivalencePartitionPanel.border.title"))); // NOI18N
        insertedEquivalencePartitionPanel.setName("insertedEquivalencePartitionPanel"); // NOI18N

        insertedEquivalencePartitionScrollPane.setName("insertedEquivalencePartitionScrollPane"); // NOI18N

        equivalencePartitionList.setName("equivalencePartitionList"); // NOI18N
        insertedEquivalencePartitionScrollPane.setViewportView(equivalencePartitionList);

        javax.swing.GroupLayout insertedEquivalencePartitionPanelLayout = new javax.swing.GroupLayout(insertedEquivalencePartitionPanel);
        insertedEquivalencePartitionPanel.setLayout(insertedEquivalencePartitionPanelLayout);
        insertedEquivalencePartitionPanelLayout.setHorizontalGroup(
            insertedEquivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(insertedEquivalencePartitionScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 605, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        insertedEquivalencePartitionPanelLayout.setVerticalGroup(
            insertedEquivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(insertedEquivalencePartitionScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 114, javax.swing.GroupLayout.PREFERRED_SIZE)
        );

        selectedMethodNameLabel2.setText(resourceMap.getString("selectedMethodNameLabel2.text")); // NOI18N
        selectedMethodNameLabel2.setName("selectedMethodNameLabel2"); // NOI18N

        selectedMethodLabel2.setForeground(resourceMap.getColor("selectedMethodLabel2.foreground")); // NOI18N
        selectedMethodLabel2.setText(resourceMap.getString("selectedMethodLabel2.text")); // NOI18N
        selectedMethodLabel2.setName("selectedMethodLabel2"); // NOI18N

        maxModelLabel.setForeground(resourceMap.getColor("maxModelLabel.foreground")); // NOI18N
        maxModelLabel.setText(resourceMap.getString("maxModelLabel.text")); // NOI18N
        maxModelLabel.setName("maxModelLabel"); // NOI18N

        maxDimensionTextField.setText(resourceMap.getString("maxDimensionTextField.text")); // NOI18N
        maxDimensionTextField.setName("maxDimensionTextField"); // NOI18N

        testForModelLabel.setForeground(resourceMap.getColor("testForModelLabel.foreground")); // NOI18N
        testForModelLabel.setText(resourceMap.getString("testForModelLabel.text")); // NOI18N
        testForModelLabel.setName("testForModelLabel"); // NOI18N

        testForModelTextField.setText(resourceMap.getString("testForModelTextField.text")); // NOI18N
        testForModelTextField.setName("testForModelTextField"); // NOI18N

        TestButton.setFont(resourceMap.getFont("TestButton.font")); // NOI18N
        TestButton.setText(resourceMap.getString("TestButton.text")); // NOI18N
        TestButton.setName("TestButton"); // NOI18N
        TestButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TestButtonActionPerformed(evt);
            }
        });

        equivalencePartitionOutputPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("equivalencePartitionOutputPanel.border.title"))); // NOI18N
        equivalencePartitionOutputPanel.setName("equivalencePartitionOutputPanel"); // NOI18N

        equivalencePartitionOutputScrollPane.setName("equivalencePartitionOutputScrollPane"); // NOI18N

        equivalencePartitionOutputEditorPane.setName("equivalencePartitionOutputEditorPane"); // NOI18N
        equivalencePartitionOutputScrollPane.setViewportView(equivalencePartitionOutputEditorPane);

        javax.swing.GroupLayout equivalencePartitionOutputPanelLayout = new javax.swing.GroupLayout(equivalencePartitionOutputPanel);
        equivalencePartitionOutputPanel.setLayout(equivalencePartitionOutputPanelLayout);
        equivalencePartitionOutputPanelLayout.setHorizontalGroup(
            equivalencePartitionOutputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(equivalencePartitionOutputScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 605, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        equivalencePartitionOutputPanelLayout.setVerticalGroup(
            equivalencePartitionOutputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(equivalencePartitionOutputScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 119, javax.swing.GroupLayout.PREFERRED_SIZE)
        );

        bitwidthLabel.setForeground(resourceMap.getColor("bitwidthLabel.foreground")); // NOI18N
        bitwidthLabel.setText(resourceMap.getString("bitwidthLabel.text")); // NOI18N
        bitwidthLabel.setName("bitwidthLabel"); // NOI18N

        bitwidthTextField.setText(resourceMap.getString("bitwidthTextField.text")); // NOI18N
        bitwidthTextField.setName("bitwidthTextField"); // NOI18N

        addEquivalencePartitionButton.setFont(resourceMap.getFont("addEquivalencePartitionButton.font")); // NOI18N
        addEquivalencePartitionButton.setText(resourceMap.getString("addEquivalencePartitionButton.text")); // NOI18N
        addEquivalencePartitionButton.setName("addEquivalencePartitionButton"); // NOI18N
        addEquivalencePartitionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addEquivalencePartitionButtonActionPerformed(evt);
            }
        });

        jButton7.setIcon(resourceMap.getIcon("jButton7.icon")); // NOI18N
        jButton7.setBorder(null);
        jButton7.setBorderPainted(false);
        jButton7.setContentAreaFilled(false);
        jButton7.setName("jButton7"); // NOI18N

        jButton8.setIcon(resourceMap.getIcon("jButton8.icon")); // NOI18N
        jButton8.setToolTipText(resourceMap.getString("jButton8.toolTipText")); // NOI18N
        jButton8.setBorderPainted(false);
        jButton8.setContentAreaFilled(false);
        jButton8.setName("jButton8"); // NOI18N

        equivalencePartitionNameLabel.setForeground(resourceMap.getColor("equivalencePartitionNameLabel.foreground")); // NOI18N
        equivalencePartitionNameLabel.setText(resourceMap.getString("equivalencePartitionNameLabel.text")); // NOI18N
        equivalencePartitionNameLabel.setName("equivalencePartitionNameLabel"); // NOI18N

        equivalencePartitionNameTextField.setFont(resourceMap.getFont("equivalencePartitionNameTextField.font")); // NOI18N
        equivalencePartitionNameTextField.setText(resourceMap.getString("equivalencePartitionNameTextField.text")); // NOI18N
        equivalencePartitionNameTextField.setName("equivalencePartitionNameTextField"); // NOI18N

        javax.swing.GroupLayout equivalencePartitionPanelLayout = new javax.swing.GroupLayout(equivalencePartitionPanel);
        equivalencePartitionPanel.setLayout(equivalencePartitionPanelLayout);
        equivalencePartitionPanelLayout.setHorizontalGroup(
            equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(equivalencePartitionOutputPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(insertedEquivalencePartitionPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(addEquivalencePartitionButton)
                            .addComponent(deleteEquivalencePartitionButton, javax.swing.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)
                            .addComponent(modifyEquivalencePartitionButton, javax.swing.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)))
                    .addComponent(equivalencePartitionInputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addComponent(selectedMethodLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 108, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(selectedMethodNameLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 419, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 122, Short.MAX_VALUE)
                        .addComponent(jButton7)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton8, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addComponent(equivalencePartitionNameLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 216, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(equivalencePartitionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 251, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addComponent(maxModelLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(4, 4, 4)
                        .addComponent(maxDimensionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 52, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(testForModelLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 166, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(testForModelTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(36, 36, 36)
                        .addComponent(bitwidthLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(bitwidthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(50, 50, 50)
                        .addComponent(TestButton, javax.swing.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)))
                .addGap(30, 30, 30))
        );

        equivalencePartitionPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {bitwidthTextField, maxDimensionTextField, testForModelTextField});

        equivalencePartitionPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {TestButton, addEquivalencePartitionButton, deleteEquivalencePartitionButton, modifyEquivalencePartitionButton});

        equivalencePartitionPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jButton7, jButton8});

        equivalencePartitionPanelLayout.setVerticalGroup(
            equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, equivalencePartitionPanelLayout.createSequentialGroup()
                                .addGap(1, 1, 1)
                                .addComponent(jButton8, javax.swing.GroupLayout.DEFAULT_SIZE, 37, Short.MAX_VALUE))
                            .addComponent(jButton7, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 38, Short.MAX_VALUE))
                        .addGap(8, 8, 8))
                    .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(selectedMethodNameLabel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(selectedMethodLabel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 42, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(equivalencePartitionNameLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(equivalencePartitionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(equivalencePartitionInputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addComponent(equivalencePartitionOutputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(insertedEquivalencePartitionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED))
                    .addGroup(equivalencePartitionPanelLayout.createSequentialGroup()
                        .addComponent(addEquivalencePartitionButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 200, Short.MAX_VALUE)
                        .addComponent(deleteEquivalencePartitionButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(modifyEquivalencePartitionButton)
                        .addGap(43, 43, 43)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(equivalencePartitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(maxDimensionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(maxModelLabel)
                    .addComponent(testForModelTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(bitwidthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(testForModelLabel)
                    .addComponent(bitwidthLabel)
                    .addComponent(TestButton))
                .addGap(38, 38, 38))
        );

        equivalencePartitionInputPanel.getAccessibleContext().setAccessibleName(resourceMap.getString("jPanel8.AccessibleContext.accessibleName")); // NOI18N
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jButton8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        tabbedPane.addTab(resourceMap.getString("equivalencePartitionPanel.TabConstraints.tabTitle"), equivalencePartitionPanel); // NOI18N

        testPanel.setName("testPanel"); // NOI18N

        selectedMethodNameLabel3.setText(resourceMap.getString("selectedMethodNameLabel3.text")); // NOI18N
        selectedMethodNameLabel3.setName("selectedMethodNameLabel3"); // NOI18N

        ReportEntryComboBox.setFont(new java.awt.Font("Tahoma", 0, 10)); // NOI18N
        ReportEntryComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "DEBUG", "VERBOSE", "DEFAULT", "WARNING", "ERROR" }));
        ReportEntryComboBox.setSelectedIndex(2);
        ReportEntryComboBox.setName("ReportEntryComboBox"); // NOI18N
        ReportEntryComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ReportEntryComboBoxActionPerformed(evt);
            }
        });

        outputPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("outputPanel.border.title"))); // NOI18N
        outputPanel.setName("outputPanel"); // NOI18N

        outputScrollPane.setAutoscrolls(true);
        outputScrollPane.setFocusTraversalPolicyProvider(true);
        outputScrollPane.setName("outputScrollPane"); // NOI18N

        outputTextPane.setBorder(null);
        outputTextPane.setName("outputTextPane"); // NOI18N
        outputScrollPane.setViewportView(outputTextPane);

        javax.swing.GroupLayout outputPanelLayout = new javax.swing.GroupLayout(outputPanel);
        outputPanel.setLayout(outputPanelLayout);
        outputPanelLayout.setHorizontalGroup(
            outputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(outputScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 664, Short.MAX_VALUE)
        );
        outputPanelLayout.setVerticalGroup(
            outputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(outputScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 471, Short.MAX_VALUE)
        );

        selectedMethodLabel3.setForeground(resourceMap.getColor("selectedMethodLabel3.foreground")); // NOI18N
        selectedMethodLabel3.setText(resourceMap.getString("selectedMethodLabel3.text")); // NOI18N
        selectedMethodLabel3.setName("selectedMethodLabel3"); // NOI18N

        outputMessageLevelLabel.setForeground(resourceMap.getColor("outputMessageLevelLabel.foreground")); // NOI18N
        outputMessageLevelLabel.setText(resourceMap.getString("outputMessageLevelLabel.text")); // NOI18N
        outputMessageLevelLabel.setName("outputMessageLevelLabel"); // NOI18N

        jButton9.setIcon(resourceMap.getIcon("jButton9.icon")); // NOI18N
        jButton9.setBorder(null);
        jButton9.setBorderPainted(false);
        jButton9.setContentAreaFilled(false);
        jButton9.setName("jButton9"); // NOI18N

        stopTestButton.setFont(new java.awt.Font("Tahoma", 0, 10));
        stopTestButton.setText(resourceMap.getString("stopTestButton.text")); // NOI18N
        stopTestButton.setName("stopTestButton"); // NOI18N
        stopTestButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopTestButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout testPanelLayout = new javax.swing.GroupLayout(testPanel);
        testPanel.setLayout(testPanelLayout);
        testPanelLayout.setHorizontalGroup(
            testPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(testPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(testPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(testPanelLayout.createSequentialGroup()
                        .addComponent(outputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())
                    .addGroup(testPanelLayout.createSequentialGroup()
                        .addGroup(testPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(testPanelLayout.createSequentialGroup()
                                .addComponent(outputMessageLevelLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 148, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(64, 64, 64)
                                .addComponent(ReportEntryComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 101, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(58, 58, 58)
                                .addComponent(stopTestButton, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(testPanelLayout.createSequentialGroup()
                                .addComponent(selectedMethodLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 118, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(selectedMethodNameLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 306, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 225, Short.MAX_VALUE)
                                .addComponent(jButton9)))
                        .addGap(61, 61, 61))))
        );
        testPanelLayout.setVerticalGroup(
            testPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(testPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(testPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(testPanelLayout.createSequentialGroup()
                        .addComponent(jButton9)
                        .addGap(13, 13, 13))
                    .addComponent(selectedMethodLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(selectedMethodNameLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(testPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(ReportEntryComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(outputMessageLevelLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(stopTestButton))
                .addGap(8, 8, 8)
                .addComponent(outputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(37, 37, 37))
        );

        jButton9.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        tabbedPane.addTab(resourceMap.getString("testPanel.TabConstraints.tabTitle"), testPanel); // NOI18N

        tabbedPane.setSelectedComponent(InputPanel);

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(statusPanelLayout.createSequentialGroup()
                        .addGap(24, 24, 24)
                        .addComponent(statusMessageLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 440, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(165, 165, 165)
                        .addComponent(statusAnimationLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, statusPanelLayout.createSequentialGroup()
                        .addGap(14, 14, 14)
                        .addComponent(tabbedPane, javax.swing.GroupLayout.DEFAULT_SIZE, 756, Short.MAX_VALUE)))
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, statusPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(tabbedPane, javax.swing.GroupLayout.PREFERRED_SIZE, 648, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(progressBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(statusMessageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(statusAnimationLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openMenuItemActionPerformed
			methodSignaturelistModel.removeAllElements();
			tabbedPane.setSelectedIndex(0);
			codiceEditorPane.setText("");
			javax.swing.JFileChooser fileChooser=new javax.swing.JFileChooser();
			fileChooser.setDialogTitle("Select Class File");
			fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			javax.swing.filechooser.FileFilter filter=new javax.swing.filechooser.FileFilter() {

			@Override
			public boolean accept(File file) {
				String filename = file.toString();
				String ext = filename.substring(filename.lastIndexOf('.')+1, filename.length());
				if(ext.equals("class") || file.isDirectory())return true;
				else return false;
			}

			@Override
			public String getDescription() {
				return (".class");
			}
			};
			fileChooser.setFileFilter(filter);           
			File path=new File(Config.TESTS_DIR);
			fileChooser.setCurrentDirectory(path);
            int retval=fileChooser.showOpenDialog(null);
			if (retval == javax.swing.JFileChooser.APPROVE_OPTION) {
				file = fileChooser.getSelectedFile();
                new JADManager(this).displayDecompiledMethod(file.getAbsolutePath(),"");
            } 
    }//GEN-LAST:event_openMenuItemActionPerformed

    private void modifyEquivalencePartitionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_modifyEquivalencePartitionButtonActionPerformed
        int index=equivalencePartitionList.getSelectedIndex();
        if(equivalencePartitionList.getSelectedValue()!=null && equivalencePartitionList.getMinSelectionIndex()==equivalencePartitionList.getMaxSelectionIndex()){
            equivalencePartitionInputEditorPane.setText(equivalencePartitionInputArray.get(index));
			equivalencePartitionOutputEditorPane.setText(equivalencePartitionOutputArray.get(index));
			equivalencePartitionNameTextField.setText(equivalencePartitionNameArray.get(index));
			maxDimensionTextField.setText(maxModelSizeArray.get(index).toString());
			testForModelTextField.setText(maxNumberOfTestArray.get(index).toString());
			bitwidthTextField.setText(bitwidthArray.get(index).toString());
            equivalencePartitionlistModel.remove(index);
            equivalencePartitionInputArray.remove(index);
			equivalencePartitionOutputArray.remove(index);
			maxModelSizeArray.remove(index);
			maxNumberOfTestArray.remove(index);
			bitwidthArray.remove(index);
			equivalencePartitionNameArray.remove(index);
            insertedEquivalencePartitions--;
        } else if(equivalencePartitionList.getMinSelectionIndex()==-1){
            JOptionPane.showMessageDialog(null,"Select 1 Equivalence Class To Modify","Warning",JOptionPane.WARNING_MESSAGE);
        } else if(equivalencePartitionList.getMinSelectionIndex()!=equivalencePartitionList.getMaxSelectionIndex()){
            JOptionPane.showMessageDialog(null,"Select Only 1 Equivalence Class To Modify","Warning",JOptionPane.WARNING_MESSAGE);
        }
}//GEN-LAST:event_modifyEquivalencePartitionButtonActionPerformed

    private void addEquivalencePartitionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addEquivalencePartitionButtonActionPerformed
		if(!((equivalencePartitionOutputEditorPane.getText().equals("")))){
			try{
			Integer.parseInt(maxDimensionTextField.getText());
			Integer.parseInt(testForModelTextField.getText());
			Integer.parseInt(bitwidthTextField.getText());			
			equivalencePartitionInputArray.add(insertedEquivalencePartitions,equivalencePartitionInputEditorPane.getText());
			equivalencePartitionInputEditorPane.setText("");
			equivalencePartitionOutputArray.add(insertedEquivalencePartitions,equivalencePartitionOutputEditorPane.getText());
			equivalencePartitionOutputEditorPane.setText("");
			equivalencePartitionNameArray.add(insertedEquivalencePartitions,equivalencePartitionNameTextField.getText());
			equivalencePartitionNameTextField.setText("");
			maxModelSizeArray.add(insertedEquivalencePartitions,Integer.parseInt(maxDimensionTextField.getText()));
			maxDimensionTextField.setText("");
			maxNumberOfTestArray.add(insertedEquivalencePartitions,Integer.parseInt(testForModelTextField.getText()));
			testForModelTextField.setText("");
			bitwidthArray.add(insertedEquivalencePartitions,Integer.parseInt(bitwidthTextField.getText()));
			bitwidthTextField.setText("");
			equivalencePartitionList.setCellRenderer(new JTextAreaRenderer(equivalencePartitionList));
			equivalencePartitionList.setModel(equivalencePartitionlistModel);
			equivalencePartitionlistModel.addElement("------------------------------------------------------------------------\n"+"Equivalence Partition: "+ equivalencePartitionNameArray.get(insertedEquivalencePartitions)+"\n"+
				("------------------------------------------------------------------------\n")+"Input:\n"+equivalencePartitionInputArray.get(insertedEquivalencePartitions)+"\n"+"output:\n"+equivalencePartitionOutputArray.get(insertedEquivalencePartitions)+
				"\n"+"Max Model Size: "+ maxModelSizeArray.get(insertedEquivalencePartitions)+"     Test For Model: "+maxNumberOfTestArray.get(insertedEquivalencePartitions)+"      Bitwidth: "+bitwidthArray.get(insertedEquivalencePartitions));
			insertedEquivalencePartitions++;
			}
			catch (NumberFormatException ex){
				JOptionPane.showMessageDialog(null,"Incompatible Type:Required int","Warning",JOptionPane.WARNING_MESSAGE);
				
			}
		}
		else{JOptionPane.showMessageDialog(null,"Insert At Least 1 Output Equivalence Partition","Warning",JOptionPane.WARNING_MESSAGE);}
}//GEN-LAST:event_addEquivalencePartitionButtonActionPerformed

    private void deleteEquivalencePartitionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteEquivalencePartitionButtonActionPerformed
        if(equivalencePartitionList.getSelectedValue()!=null && equivalencePartitionList.getMinSelectionIndex()==equivalencePartitionList.getMaxSelectionIndex()){
            int index=equivalencePartitionList.getSelectedIndex();
            equivalencePartitionlistModel.remove(index);
            equivalencePartitionInputArray.remove(index);
			equivalencePartitionOutputArray.remove(index);
			maxModelSizeArray.remove(index);
			maxNumberOfTestArray.remove(index);
			bitwidthArray.remove(index);
			equivalencePartitionNameArray.remove(index);
            insertedEquivalencePartitions--;
        } 
		else if(equivalencePartitionList.getMinSelectionIndex()==-1){
            JOptionPane.showMessageDialog(null,"Select 1 Equivalence Partition To Remove","Warning",JOptionPane.WARNING_MESSAGE);
        } 
		else if(equivalencePartitionList.getMinSelectionIndex()!=equivalencePartitionList.getMaxSelectionIndex()){
            JOptionPane.showMessageDialog(null,"Select Only 1 Equivalence Partition To Remove","Warning",JOptionPane.WARNING_MESSAGE);
        }
}//GEN-LAST:event_deleteEquivalencePartitionButtonActionPerformed

    private void clearPreconditionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearPreconditionButtonActionPerformed
		preconditionEditorPane.setText("");
}//GEN-LAST:event_clearPreconditionButtonActionPerformed

private void ReportEntryComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ReportEntryComboBoxActionPerformed
		String livelloSelezionato=null;
		livelloSelezionato=(String)ReportEntryComboBox.getSelectedItem();
		if(livelloSelezionato.equals("DEBUG")){reportLevel=1;}
		else if(livelloSelezionato.equals("VERBOSE")){reportLevel=2;}
		else if(livelloSelezionato.equals("DEFAULT")){reportLevel=3;}
		else if(livelloSelezionato.equals("WARNING")){reportLevel=4;}
		else if(livelloSelezionato.equals("ERROR")){reportLevel=5;}
}//GEN-LAST:event_ReportEntryComboBoxActionPerformed

private void TestButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TestButtonActionPerformed
	new ReportManager(this).clear();
	progressBar.setValue(0);
	if(methodSignatureList.getSelectedIndex()!=-1 && insertedEquivalencePartitions!=0){
		try{			
			tabbedPane.setSelectedIndex(4);
			ReportManager repMan=new ReportManager(this);
			final Test test = new Test("java", "alloy", repMan);
				thr  = new Thread(){
				public void run(){
					try {
						List<EquivalencePartition> parts = new ArrayList<EquivalencePartition>();
						for(int i=0;i<equivalencePartitionOutputArray.size();i++){
							EquivalencePartition eq = new EquivalencePartition(equivalencePartitionNameArray.get(i), equivalencePartitionInputArray.get(i), equivalencePartitionOutputArray.get(i), maxModelSizeArray.get(i),maxNumberOfTestArray.get(i),bitwidthArray.get(i));
							parts.add(eq);
						}
						List<Function> funcs = test.searchFunctions(file.getAbsolutePath());
						test.executeTest(preconditionEditorPane.getText(), parts,functionEditorPane.getText(), funcs.get(methodSignatureList.getSelectedIndex()));
						progressBar.setVisible(false);
						statusAnimationLabel.setIcon(idleIcon);
						statusMessageLabel.setText("");
						thr2.interrupt();
						return;
					}catch (Exception e) {
						test.getReporter().log(Level.ERROR, "Exception was throw", e);
						progressBar.setVisible(false);
						statusMessageLabel.setText("");
						statusAnimationLabel.setIcon(idleIcon);
						thr2.interrupt();
					}
				}
				};
				thr.start();
				thr2 = new Thread(){
				public void run(){
				try {
					while(true){	 
						busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
						statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
						statusMessageLabel.setText("Jamote Is Testing Selected Method...");
						sleep(50);
					}
				}catch (InterruptedException ex) {		 
				}
			    }};
			   thr2.start();
		}
		catch(JamoteException e){
			JOptionPane.showMessageDialog(null,e.toString(),"Thrown Exception",JOptionPane.ERROR_MESSAGE);
		}
	}
	else{
		if(methodSignatureList.getSelectedIndex()==-1){
			JOptionPane.showMessageDialog(null,"Select A Method To Test","Warning",JOptionPane.WARNING_MESSAGE);
		}
		else if(insertedEquivalencePartitions==0){
			JOptionPane.showMessageDialog(null,"Insert At Least An EquivalencePartitionOutput","Warning",JOptionPane.WARNING_MESSAGE);
		}
	}
}//GEN-LAST:event_TestButtonActionPerformed

private void clearFunctionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearFunctionButtonActionPerformed
	functionEditorPane.setText("");
}//GEN-LAST:event_clearFunctionButtonActionPerformed


private void openClassInPackageMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openClassInPackageMenuItemActionPerformed
	progressBar.setValue(0);
	insertedEquivalencePartitions=0;
	equivalencePartitionlistModel.removeAllElements();
	methodSignaturelistModel.removeAllElements();
    equivalencePartitionInputArray.removeAll(equivalencePartitionInputArray);
	equivalencePartitionOutputArray.removeAll(equivalencePartitionInputArray);	
	tabbedPane.setSelectedIndex(0);
	codiceEditorPane.setText("");
	JFileChooser fileChooser=new JFileChooser();
	fileChooser.setDialogTitle("Select Class File");
	fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	javax.swing.filechooser.FileFilter filter=new javax.swing.filechooser.FileFilter() {

	@Override
	public boolean accept(File file) {
		String filename = file.toString();
		String ext = filename.substring(filename.lastIndexOf('.')+1, filename.length());
		if(ext.equals("class") || file.isDirectory())return true;
		else return false;
	}

	@Override
	public String getDescription() {
		return (".class");
	}
	};

	fileChooser.setFileFilter(filter);
	javax.swing.JFileChooser packageChooser=new javax.swing.JFileChooser();
	packageChooser.setDialogTitle("Select Class Package");
	packageChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	File path=new File(Config.TESTS_DIR);
	fileChooser.setCurrentDirectory(path);
	int retval=fileChooser.showOpenDialog(null);
	if (retval == javax.swing.JFileChooser.APPROVE_OPTION) {
		file = fileChooser.getSelectedFile();
	}
	int retval2=packageChooser.showOpenDialog(null);
	if (retval2 == javax.swing.JFileChooser.APPROVE_OPTION) {
		packFile = packageChooser.getSelectedFile();
		new JADManager(this).displayDecompiledMethod(file.getAbsolutePath(),packFile.getAbsolutePath());
	}                   
}//GEN-LAST:event_openClassInPackageMenuItemActionPerformed

private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
	tabbedPane.setSelectedIndex(tabbedPane.getSelectedIndex()+1);
}//GEN-LAST:event_jButton2ActionPerformed

private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
	tabbedPane.setSelectedIndex(tabbedPane.getSelectedIndex()-1);
}//GEN-LAST:event_jButton3ActionPerformed

private void stopTestButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopTestButtonActionPerformed
    thr.interrupt();
	thr2.interrupt();
    progressBar.setVisible(false);
	statusMessageLabel.setText("");
	statusAnimationLabel.setIcon(idleIcon);
}//GEN-LAST:event_stopTestButtonActionPerformed

private void methodSignatureListMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_methodSignatureListMousePressed
	selectedMethodNameLabel.setText("");
    selectedMethodNameLabel2.setText("");
    selectedMethodNameLabel3.setText("");
    selectedMethodNameLabel4.setText("");
	if(methodSignatureList.getSelectedValue()!=null && methodSignatureList.getMinSelectionIndex()==methodSignatureList.getMaxSelectionIndex()){
		metodoSelezionato=methodSignatureList.getSelectedValue();
		selectedMethodNameLabel.setText(metodoSelezionato.toString());
		selectedMethodNameLabel2.setText(metodoSelezionato.toString());
		selectedMethodNameLabel3.setText(metodoSelezionato.toString());
		selectedMethodNameLabel4.setText(metodoSelezionato.toString());
	}      
}//GEN-LAST:event_methodSignatureListMousePressed

private void saveAlloyFileMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAlloyFileMenuItemActionPerformed
	File xmlFile;
	JFileChooser saveFileChooser=new JFileChooser();
	int retval=saveFileChooser.showSaveDialog(null);
	if (retval == javax.swing.JFileChooser.APPROVE_OPTION) {
		xmlFile = saveFileChooser.getSelectedFile();
		XmlFileManager xfm=new XmlFileManager(blackBox);
		xfm.createXmlFile(xmlFile);
    }
	
}//GEN-LAST:event_saveAlloyFileMenuItemActionPerformed

private void openAlloyFileMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openAlloyFileMenuItemActionPerformed
	File xmlFile;
	JFileChooser loadFileChooser=new JFileChooser();
	int retval=loadFileChooser.showOpenDialog(null);
	if (retval == javax.swing.JFileChooser.APPROVE_OPTION) {
		xmlFile = loadFileChooser.getSelectedFile();
		new XmlFileManager(blackBox).loadXmlFile(xmlFile);
	}
}//GEN-LAST:event_openAlloyFileMenuItemActionPerformed

protected void resetGUIvalues(){
		progressBar.setValue(0);
		insertedEquivalencePartitions=0;
		equivalencePartitionlistModel.removeAllElements();
		equivalencePartitionInputArray.clear();
		equivalencePartitionOutputArray.clear();
		maxModelSizeArray.clear();
		maxNumberOfTestArray.clear();
		bitwidthArray.clear();
		equivalencePartitionNameArray.clear();
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel InputPanel;
    protected javax.swing.JComboBox ReportEntryComboBox;
    private javax.swing.JButton TestButton;
    private javax.swing.JButton addEquivalencePartitionButton;
    private javax.swing.JLabel bitwidthLabel;
    protected javax.swing.JTextField bitwidthTextField;
    private javax.swing.JButton clearFunctionButton;
    private javax.swing.JButton clearPreconditionButton;
    protected javax.swing.JEditorPane codiceEditorPane;
    private javax.swing.JScrollPane codiceEditorScrollPane;
    private javax.swing.JButton deleteEquivalencePartitionButton;
    protected javax.swing.JEditorPane equivalencePartitionInputEditorPane;
    private javax.swing.JPanel equivalencePartitionInputPanel;
    private javax.swing.JScrollPane equivalencePartitionInputScrollPane;
    protected javax.swing.JList equivalencePartitionList;
    private javax.swing.JLabel equivalencePartitionNameLabel;
    protected javax.swing.JTextField equivalencePartitionNameTextField;
    protected javax.swing.JEditorPane equivalencePartitionOutputEditorPane;
    private javax.swing.JPanel equivalencePartitionOutputPanel;
    private javax.swing.JScrollPane equivalencePartitionOutputScrollPane;
    private javax.swing.JPanel equivalencePartitionPanel;
    private javax.swing.JPanel extractMethodsPanel;
    protected javax.swing.JEditorPane functionEditorPane;
    private javax.swing.JPanel functionPanel;
    private javax.swing.JScrollPane functionScrollPane;
    private javax.swing.JPanel insertFunctionPanel;
    private javax.swing.JPanel insertPreconditionPanel;
    private javax.swing.JScrollPane insertPreconditionScrollPane;
    private javax.swing.JPanel insertedEquivalencePartitionPanel;
    private javax.swing.JScrollPane insertedEquivalencePartitionScrollPane;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JButton jButton7;
    private javax.swing.JButton jButton8;
    private javax.swing.JButton jButton9;
    protected javax.swing.JTextField maxDimensionTextField;
    private javax.swing.JLabel maxModelLabel;
    private javax.swing.JMenuBar menuBar;
    protected javax.swing.JList methodSignatureList;
    private javax.swing.JScrollPane methodSignatureScrollPane;
    private javax.swing.JButton modifyEquivalencePartitionButton;
    private javax.swing.JMenuItem openAlloyFileMenuItem;
    private javax.swing.JMenuItem openClassInPackageMenuItem;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JLabel outputMessageLevelLabel;
    private javax.swing.JPanel outputPanel;
    private javax.swing.JScrollPane outputScrollPane;
    protected javax.swing.JTextPane outputTextPane;
    protected javax.swing.JEditorPane preconditionEditorPane;
    private javax.swing.JPanel preconditionPanel;
    protected javax.swing.JProgressBar progressBar;
    private javax.swing.JMenuItem saveAlloyFileMenuItem;
    private javax.swing.JLabel selectedMethodLabel;
    private javax.swing.JLabel selectedMethodLabel2;
    private javax.swing.JLabel selectedMethodLabel3;
    private javax.swing.JLabel selectedMethodLabel4;
    private javax.swing.JLabel selectedMethodNameLabel;
    private javax.swing.JLabel selectedMethodNameLabel2;
    private javax.swing.JLabel selectedMethodNameLabel3;
    private javax.swing.JLabel selectedMethodNameLabel4;
    private javax.swing.JPanel sourceCodePanel;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JButton stopTestButton;
    protected javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JLabel testForModelLabel;
    protected javax.swing.JTextField testForModelTextField;
    private javax.swing.JPanel testPanel;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;
    private Object metodoSelezionato;
    private JDialog aboutBox;
    protected javax.swing.DefaultListModel methodSignaturelistModel = new DefaultListModel();
    protected javax.swing.DefaultListModel equivalencePartitionlistModel = new DefaultListModel();
    protected int insertedEquivalencePartitions=0;
    protected ArrayList<String> equivalencePartitionInputArray=new ArrayList<String>();
	protected ArrayList<String> equivalencePartitionOutputArray=new ArrayList<String>();
	protected ArrayList<String> equivalencePartitionNameArray=new ArrayList<String>();
	protected ArrayList<Integer> maxModelSizeArray=new ArrayList<Integer>();
	protected ArrayList<Integer> maxNumberOfTestArray=new ArrayList<Integer>();
	protected ArrayList<Integer> bitwidthArray=new ArrayList<Integer>();
    protected int reportLevel=3;
	protected File file;
    private File packFile;
	protected int value=0;
    private Thread thr;
	private Thread thr2;
	protected int numberOfTest=1;
	protected String[] parseNumberOfTest;
	protected BlackBoxTestGuiView blackBox = this;

	  
}