/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */

package it.dis.uniroma1.jamote.gui;

import it.dis.uniroma1.jamote.utils.ReportEntry;
import it.dis.uniroma1.jamote.utils.ReportHandler;
import java.awt.Color;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.util.Date;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;



public class ReportManager implements ReportHandler{
	
	BlackBoxTestGuiView my_gui;
	
	public ReportManager(BlackBoxTestGuiView gui){
		my_gui = gui;
	}

	public void handleReport(ReportEntry entry) {

		Style style = null;
		if(entry.getLevel()==ReportEntry.Level.VERBOSE){
			style = my_gui.outputTextPane.addStyle("Blue", null);
			StyleConstants.setForeground(style, Color.blue);
		}
		else if(entry.getLevel()==ReportEntry.Level.ERROR){
			style = my_gui.outputTextPane.addStyle("Red", null);
			StyleConstants.setForeground(style, Color.red);
		}
		else if(entry.getLevel()==ReportEntry.Level.WARNING){
			style = my_gui.outputTextPane.addStyle("Orange", null);
			StyleConstants.setForeground(style, Color.orange);
		}
		else if(entry.getLevel()==ReportEntry.Level.DEFAULT){
			style = my_gui.outputTextPane.addStyle("Black", null);
			StyleConstants.setForeground(style, Color.black);
		}
		else if(entry.getLevel()==ReportEntry.Level.DEBUG){
			style = my_gui.outputTextPane.addStyle("Gray", null);
			StyleConstants.setForeground(style, Color.gray);
		}
		try {
			StringWriter exc = new StringWriter();
			if(entry.getThrown() != null)
			{
				entry.getThrown().printStackTrace(new PrintWriter(exc));
			}
		   StyledDocument doc = my_gui.outputTextPane.getStyledDocument();
			   if(entry.getLevel().id >= my_gui.reportLevel){
					if(my_gui.ReportEntryComboBox.getSelectedItem().equals("DEBUG")){
						doc.insertString(doc.getLength(),"Message:\t"+entry.getMessage()+"\n", style);
						doc.insertString(doc.getLength(),"Method:\t"+entry.getSourceMethodName()+"\n", style);
						doc.insertString(doc.getLength(),"Exception:\t"+exc.getBuffer().toString()+"\n", style);
						doc.insertString(doc.getLength(),"Time:\t"+DateFormat.getInstance().format(new Date(entry.getTimeMillis()))+"\n", style);
						doc.insertString(doc.getLength(),"Line:\t"+Integer.toString(entry.getSourceLineNumber())+"\n", style);
						doc.insertString(doc.getLength(),"File:\t"+entry.getSourceFileName()+"\n", style);
						doc.insertString(doc.getLength(),"\n", style); 
					}
					else{
						doc.insertString(doc.getLength(),"Message:\t"+entry.getMessage()+"\n", style);
						if(entry.getThrown()!=null){
							doc.insertString(doc.getLength(),"Exception:\t"+exc.getBuffer().toString()+"\n", style);
						}
					}
			   }
			   my_gui.outputTextPane.setCaretPosition(doc.getLength());
			   if(entry.getMessage().startsWith("Generating models")){
					my_gui.progressBar.setVisible(true);	
			   }
			   	  	
			   if(entry.getMessage().startsWith("Generation of models completed.")){
					my_gui.parseNumberOfTest=entry.getMessage().split("\\s");
					my_gui.numberOfTest=Integer.parseInt(my_gui.parseNumberOfTest[4]);
			   }
			   if(entry.getMessage().startsWith("Testing with input")){
					my_gui.value=my_gui.progressBar.getValue();
					
					my_gui.progressBar.setValue(my_gui.value + my_gui.progressBar.getMaximum()/(my_gui.maxNumberOfTestArray.size()*my_gui.numberOfTest));
			   }
			   if(entry.getLevel()==ReportEntry.Level.ERROR){my_gui.progressBar.setValue(0);}

		} catch (BadLocationException e) {
			JOptionPane.showMessageDialog(null,e.toString(),"Thrown Exception",JOptionPane.ERROR_MESSAGE);
		  }
	}

	
	
    public void clear() {
       my_gui.outputTextPane.setText("");
    }

}
