/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */


package it.dis.uniroma1.jamote.utils;

import it.dis.uniroma1.jamote.utils.ReportEntry.Level;


/**
 * Rappresenta un insieme di log records.
 *
 */
public class Reporter {
	protected ReportHandler handler;
	
	public Reporter(ReportHandler handler){
		this.handler = handler;
	}
	
	/**
	 * Add a log entry
	 * @param level
	 * @param message
	 * @param thrown
	 */
	public void log(Level level, String message, Throwable thrown){
		String 	sourceClassName = null;
		String 	sourceMethodName  = null;
		int		sourceLineNumber  = 0;
		String 	sourceFileName  = null;
		long	timeMillis;
		
		timeMillis = System.currentTimeMillis();
		
		//Prendo il nome del metodo e della classe chiamante
		StackTraceElement stack[] = Thread.currentThread().getStackTrace();
		boolean takeCurrent = false;
		for(StackTraceElement s : stack){
			takeCurrent = ! (s.getClassName().equals(this.getClass().getName()) || 
					(s.getMethodName().equals("getStackTrace")));
			if(takeCurrent)
			{
				sourceClassName = s.getClassName();
				sourceMethodName = s.getMethodName();
				sourceLineNumber = s.getLineNumber();
				sourceFileName = s.getFileName();
				break;
			}
		}
			
		handler.handleReport(new ReportEntry(level, message, sourceClassName, sourceMethodName,
				sourceFileName, sourceLineNumber, thrown, timeMillis));
	}
	
	/**
	 * Add a log entry
	 * @param level
	 * @param message
	 * @param thrown
	 */
	public void log(Level level, String message){
		log(level, message, null);
	}
}
