/*
 * Jamote - JAva MOdel-based black box TEsting (http://www.assembla.com/wiki/show/jamote)
 * Copyright (C) 2008 Jamote Team (http://www.assembla.com/spaces/jamote/team)
 * 
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Jamote utilizes several third-party packages whose code may
 * be distributed under a different license, see the file named LICENSE for
 * more details.
 */


package it.dis.uniroma1.jamote.gui;

import it.dis.uniroma1.jamote.core.JamoteException;
import it.dis.uniroma1.jamote.core.Test;
import it.dis.uniroma1.jamote.core.language.Function;
import it.dis.uniroma1.jamote.utils.Config;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Modifier;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import javax.swing.JOptionPane;



public class JADManager {
	
	BlackBoxTestGuiView my_gui;
	public JADManager(BlackBoxTestGuiView gui){
		my_gui=gui;
	}
	public JADManager(){
	}    
	protected String extractJad() throws IOException{
		final String tmpDir = System.getProperty("java.io.tmpdir") + "/Jamote";
		String os = System.getProperty("os.name").split("\\s")[0].toLowerCase();
		String[] filename;
		if(os.equals("linux"))
			filename = new String[]{"lib","linux","jad"};
		else if(os.equals("windows"))
			filename = new String[]{"lib","windows","jad.exe"};
		else if(os.equals("mac"))
			filename = new String[]{"lib","mac","jad"};
		else return null;	
		File jadf = new File(tmpDir+"/"+filename[0]+"/"+filename[1]+"/"+filename[2]);
		if(jadf.exists())
			return jadf.getAbsolutePath();
		try{	
			String jarPath =
			this.getClass().getProtectionDomain().getCodeSource().getLocation().toURI().getPath();
			JarFile jarf = new JarFile(jarPath);
			JarEntry je =
			jarf.getJarEntry(filename[0]+"/"+filename[1]+"/"+filename[2]);
			jadf.getParentFile().mkdirs();
			jadf.createNewFile();
			BufferedInputStream bis = new BufferedInputStream(jarf.getInputStream(je));
			BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(jadf));
			byte[] buffer = new byte[1024];
			int len;
			while((len = bis.read(buffer)) >= 0)
				bos.write(buffer, 0, len);
				bis.close();
				bos.close();
		}catch(URISyntaxException ex){}
		return jadf.getAbsolutePath();
	}
	
	protected void displayDecompiledMethod(String filePath,String packagePath){
		String neededPath="";
		try{

			String differencePath=filePath.substring(packagePath.length()+1,filePath.length());
			neededPath=(packagePath.equals("")? filePath:packagePath+"/" +(differencePath.replaceAll("/",".")));
			my_gui.methodSignatureList.setModel(my_gui.methodSignaturelistModel);
			String os = System.getProperty("os.name").split("\\s")[0].toLowerCase();
			ProcessBuilder pb;
			if(new File(Config.JAD_DIR+"/"+os).exists()){
				File jadFile=new File(Config.JAD_DIR+"/"+os+"/jad");  
				jadFile.setExecutable(true,true);
				pb=new ProcessBuilder(jadFile.getAbsolutePath() ,"-ff","-p", filePath);
			}
			else{
				String dir = this.extractJad();
				File jadFile=new File(dir);
				jadFile.setExecutable(true,true);
				pb=new ProcessBuilder(jadFile.getAbsolutePath(),"-ff","-p", filePath);
			}
			Process p=pb.start();            
			BufferedReader in =new BufferedReader(new InputStreamReader(p.getInputStream()));
			String line;
			String sourceCode="";
			while ((line = in.readLine()) != null) {
				if(!line.startsWith("//")){
					sourceCode=my_gui.codiceEditorPane.getText().concat(line).concat("\n");
					my_gui.codiceEditorPane.setText(sourceCode);
				}
			}  
		}
		catch(IOException e){
			JOptionPane.showMessageDialog(null,e.toString(),"Error",JOptionPane.ERROR_MESSAGE);
			my_gui.codiceEditorPane.setText("Error In Decompiling Class File Due To Decompiler's Limitations."+"\n"+"Source Code Will Not Be Displayed!");
		}
		finally{
			try{
				Test test= new Test("java","alloy",new ReportManager(my_gui));
				List<Function> methodList=new ArrayList<Function>();
				methodList = test.searchFunctions(neededPath);
				String guiMethod;
				for(int i = 0;i<methodList.size();i++){
					List<String> stringList=new ArrayList<String>();
					stringList = methodList.get(i).getParameterTypes();
					guiMethod=(Modifier.toString((methodList.get(i).getModifiers()))).concat(" ").concat(methodList.get(i).getReturnType()).concat(" ").concat(methodList.get(i).getName().concat("("));
					for(int j=0;j<stringList.size();j++){
						guiMethod=guiMethod.concat(stringList.get(j));
						guiMethod=guiMethod.concat(" ");
						guiMethod=guiMethod.concat("args"+j);
						guiMethod=guiMethod.concat(",");
					}
					guiMethod=(stringList.size() == 0 ? guiMethod : guiMethod.substring(0,guiMethod.length()-1));
					guiMethod=guiMethod.concat(")");
					my_gui.methodSignaturelistModel.addElement(guiMethod);
				}
			}catch(JamoteException e){
				JOptionPane.showMessageDialog(null,e.toString(),"Error",JOptionPane.ERROR_MESSAGE);
			}
		}
	}
}
